#include "SamsungIAP.h"
#include "IAP.h"
#include "base64.h"
#include "IAPJniWrapper.h"

using namespace std;
using namespace json98;

namespace samsung {

	void IAP::setOperationMode(int mode) {
		IAPJniWrapper::getInstance()->setOperationMode(mode);
	}

	void IAP::getProductDetails(const FString& products) {
		IAPJniWrapper::getInstance()->getProductDetails(TCHAR_TO_UTF8(*products));
	}

	void IAP::getOwnedList(const FString& type) {
		IAPJniWrapper::getInstance()->getOwnedList(TCHAR_TO_UTF8(*type));
	}

	void IAP::startPayment(const FString& item_id, const FString& pass_through_param, bool show_success_dialog) {
		IAPJniWrapper::getInstance()->startPayment(TCHAR_TO_UTF8(*item_id), TCHAR_TO_UTF8(*pass_through_param), show_success_dialog);
	}

    void IAP::consumePurchasedItems(const FString& item_ids) {
        IAPJniWrapper::getInstance()->consumePurchasedItems(TCHAR_TO_UTF8(*item_ids));
    }

	void IAP::setListener(IAPListener* listener) {
        IAPJniWrapper::getInstance()->setListener(listener);
	}

	void IAP::removeListener() {
        IAPJniWrapper::getInstance()->removeListener();
	}

	IAPListener* IAP::getListener() {
		return IAPJniWrapper::getInstance()->getListener();
	}

	/********************************************
	 * Data classes
	 *******************************************/
	BaseVo::BaseVo(const json98::Json& json) {

        mItemId = FString(UTF8_TO_TCHAR(json["mItemId"].string_value().c_str()));
        mItemName = FString(UTF8_TO_TCHAR(json["mItemName"].string_value().c_str()));
        mItemPrice = json["mItemPrice"].double_value();
        mItemPriceString = FString(UTF8_TO_TCHAR(json["mItemPriceString"].string_value().c_str()));
        mCurrencyUnit = FString(UTF8_TO_TCHAR(json["mCurrencyUnit"].string_value().c_str()));
        mCurrencyCode = FString(UTF8_TO_TCHAR(json["mCurrencyCode"].string_value().c_str()));
        mItemDesc = FString(UTF8_TO_TCHAR(json["mItemDesc"].string_value().c_str()));
        mType = FString(UTF8_TO_TCHAR(json["mType"].string_value().c_str()));

        string is_consumable = json["mConsumableYN"].string_value();
        if(is_consumable.compare("Y") == 0) {
            mIsConsumable = true;
        } else {
            mIsConsumable = false;
        }
	}

	ProductVo::ProductVo(const json98::Json& json):BaseVo(json) {

        mSubscriptionDurationUnit = FString(UTF8_TO_TCHAR(json["mSubscriptionDurationUnit"].string_value().c_str()));
        mSubscriptionDurationMultiplier = FString(UTF8_TO_TCHAR(json["mSubscriptionDurationMultiplier"].string_value().c_str()));
        mItemImageUrl = FString(UTF8_TO_TCHAR(json["mItemImageUrl"].string_value().c_str()));
        mItemDownloadUrl = FString(UTF8_TO_TCHAR(json["mItemDownloadUrl"].string_value().c_str()));
        mReserved1 = FString(UTF8_TO_TCHAR(json["mReserved1"].string_value().c_str()));
        mReserved2 = FString(UTF8_TO_TCHAR(json["mReserved2"].string_value().c_str()));
        mFreeTrialPeriod = FString(UTF8_TO_TCHAR(json["mFreeTrialPeriod"].string_value().c_str()));
	}

	OwnedProductVo::OwnedProductVo(const json98::Json& json):BaseVo(json) {
		
		mPaymentId = FString(UTF8_TO_TCHAR(json["mPaymentId"].string_value().c_str()));
		mPurchaseId = FString(UTF8_TO_TCHAR(json["mPurchaseId"].string_value().c_str()));
		mPurchaseDate = json["mPurchaseDate"].long_value();
		mPassThroughParam = FString(UTF8_TO_TCHAR(base64_decode(json["mPassThroughParam"].string_value()).c_str()));

		mSubscriptionEndDate = json["mSubscriptionEndDate"].long_value();
	}

	PurchaseVo::PurchaseVo(const json98::Json& json):BaseVo(json) {

		mPaymentId = FString(UTF8_TO_TCHAR(json["mPaymentId"].string_value().c_str()));
        mPurchaseId = FString(UTF8_TO_TCHAR(json["mPurchaseId"].string_value().c_str()));
        mPurchaseDate = json["mPurchaseDate"].long_value();

        mPassThroughParam = FString(UTF8_TO_TCHAR(base64_decode(json["mPassThroughParam"].string_value()).c_str()));

        mItemImageUrl = FString(UTF8_TO_TCHAR(json["mItemImageUrl" ].string_value().c_str()));
        mItemDownloadUrl = FString(UTF8_TO_TCHAR(json[ "mItemDownloadUrl" ].string_value().c_str()));
        mReserved1 = FString(UTF8_TO_TCHAR(json[ "mReserved1" ].string_value().c_str()));
        mReserved2 = FString(UTF8_TO_TCHAR(json[ "mReserved2" ].string_value().c_str()));
        mVerifyUrl = FString(UTF8_TO_TCHAR(json[ "mVerifyUrl"].string_value().c_str()));
	}

	ConsumeVo::ConsumeVo(const json98::Json& json) {

        mPurchaseId = FString(UTF8_TO_TCHAR(json["mPurchaseId"].string_value().c_str()));
        mStatusString = FString(UTF8_TO_TCHAR(json["mStatusString"].string_value().c_str()));
        mStatusCode = FString(UTF8_TO_TCHAR(json["mStatusCode"].string_value().c_str()));
	}
}