#include "SamsungIAP.h"
#include "IAPJniWrapper.h"
#include "json98.h"
#include "base64.h"
#include "Async.h"

using namespace std;
using namespace json98;

DEFINE_LOG_CATEGORY(SamsungIAP_Native);

#if PLATFORM_ANDROID

#include "Android/AndroidApplication.h"
#include "Android/AndroidJNI.h"

//Initialize JNI context
#define INIT_JAVA_METHOD(name, signature) \
if (JNIEnv* Env = FAndroidApplication::GetJavaEnv(true)) { \
    name = FJavaWrapper::FindMethod(Env, FJavaWrapper::GameActivityClassID, #name, signature, false); \
    check(name != NULL); \
} else { \
    check(0); \
}

#define DECLARE_JAVA_METHOD(name) \
static jmethodID name = NULL;

string jstring2string(JNIEnv* env, jstring jstr) {
	if (jstr == nullptr || !env) {
		return "";
	}

	const char* chars = env->GetStringUTFChars(jstr, nullptr);
	string ret(chars);
	env->ReleaseStringUTFChars(jstr, chars);

	return ret;
}

FString jstring2FString(JNIEnv* env, jstring jstr) {
	if (jstr == nullptr || !env) {
		return "";
	}

	const char* chars = env->GetStringUTFChars(jstr, nullptr);
	FString ret = FString(UTF8_TO_TCHAR(chars));
	env->ReleaseStringUTFChars(jstr, chars);

	return ret;
}

extern "C" void Java_com_epicgames_ue4_GameActivity_nativeOnGetProducts(JNIEnv *env, jobject thiz,
                                                                           jint result,
                                                                           jstring msg_,
                                                                           jstring data_) {
	string data = jstring2string(env, data_);
	FString msg = jstring2FString(env, msg_);

	AsyncTask(ENamedThreads::GameThread, [=]() {
		samsung::IAPJniWrapper::getInstance()->onGetProducts(result, msg, data);
	});
}


extern "C" void Java_com_epicgames_ue4_GameActivity_nativeOnGetOwnedProducts(JNIEnv *env, jobject thiz,
                                                                                jint result,
                                                                                jstring msg_,
                                                                                jstring data_) {
	string data = jstring2string(env, data_);
	FString msg = jstring2FString(env, msg_);

	AsyncTask(ENamedThreads::GameThread, [=]() {
		samsung::IAPJniWrapper::getInstance()->onGetOwnedProducts(result, msg, data);
	});
}


extern "C" void Java_com_epicgames_ue4_GameActivity_nativeOnPayment(JNIEnv *env, jobject thiz,
                                                                       jint result,
                                                                       jstring msg_,
                                                                       jstring data_) {
	string data = jstring2string(env, data_);
	FString msg = jstring2FString(env, msg_);

	AsyncTask(ENamedThreads::GameThread, [=]() {
		samsung::IAPJniWrapper::getInstance()->onPayment(result, msg, data);
	});
}


extern "C" void Java_com_epicgames_ue4_GameActivity_nativeOnConsumePurchasedItems(JNIEnv *env, jobject thiz,
                                                                                     jint result,
                                                                                     jstring msg_,
                                                                                     jstring data_) {
	string data = jstring2string(env, data_);
	FString msg = jstring2FString(env, msg_);

	AsyncTask(ENamedThreads::GameThread, [=]() {
		samsung::IAPJniWrapper::getInstance()->onConsumePurchasedItems(result, msg, data);
	});
}

namespace samsung {

DECLARE_JAVA_METHOD(SamsungIAP_setOperationMode);
DECLARE_JAVA_METHOD(SamsungIAP_getProductDetails);
DECLARE_JAVA_METHOD(SamsungIAP_getOwnedList);
DECLARE_JAVA_METHOD(SamsungIAP_startPayment);
DECLARE_JAVA_METHOD(SamsungIAP_consumePurchasedItems);

    static IAPJniWrapper* s_instance = nullptr;

    IAPJniWrapper* IAPJniWrapper::getInstance() {
        if(!s_instance) {
            s_instance = new IAPJniWrapper();
        }

        return s_instance;
    }

    IAPJniWrapper::IAPJniWrapper() {
        _listener = nullptr;

        INIT_JAVA_METHOD(SamsungIAP_setOperationMode, "(I)V");
        INIT_JAVA_METHOD(SamsungIAP_getProductDetails, "(Ljava/lang/String;)V");
        INIT_JAVA_METHOD(SamsungIAP_getOwnedList, "(Ljava/lang/String;)V");
        INIT_JAVA_METHOD(SamsungIAP_startPayment, "(Ljava/lang/String;Ljava/lang/String;Z)V");
        INIT_JAVA_METHOD(SamsungIAP_consumePurchasedItems, "(Ljava/lang/String;)V");
    }

    IAPJniWrapper::~IAPJniWrapper() {
    }

    void IAPJniWrapper::setOperationMode(int mode) {
        if (JNIEnv* Env = FAndroidApplication::GetJavaEnv(true)) {
            FJavaWrapper::CallVoidMethod(Env, FJavaWrapper::GameActivityThis, SamsungIAP_setOperationMode, mode);
            UE_LOG(LogAndroid, Warning, TEXT("I found the java method SamsungIAP_setOperationMode\n"));
        }
        else
        {
            UE_LOG(LogAndroid, Warning, TEXT("ERROR Could note get Java ENV\n"));
        }
    }

    void IAPJniWrapper::getProductDetails(const std::string& products) {
        if (JNIEnv* Env = FAndroidApplication::GetJavaEnv(true)) {
            jstring str = Env->NewStringUTF(products.c_str());
            FJavaWrapper::CallVoidMethod(Env, FJavaWrapper::GameActivityThis, SamsungIAP_getProductDetails, str);
            Env->DeleteLocalRef(str);
            UE_LOG(LogAndroid, Warning, TEXT("I found the java method SamsungIAP_getProductDetails\n"));
        }
        else
        {
            UE_LOG(LogAndroid, Warning, TEXT("ERROR Could note get Java ENV\n"));
        }
    }

    void IAPJniWrapper::getOwnedList(const std::string& type) {
        if (JNIEnv* Env = FAndroidApplication::GetJavaEnv(true)) {
            jstring str = Env->NewStringUTF(type.c_str());
            FJavaWrapper::CallVoidMethod(Env, FJavaWrapper::GameActivityThis, SamsungIAP_getOwnedList, str);
            Env->DeleteLocalRef(str);
            UE_LOG(LogAndroid, Warning, TEXT("I found the java method SamsungIAP_getOwnedList\n"));
        }
        else
        {
            UE_LOG(LogAndroid, Warning, TEXT("ERROR Could note get Java ENV\n"));
        }
    }

    void IAPJniWrapper::startPayment(const std::string& item_id, const std::string& pass_through_param, bool success_dialog) {
        if (JNIEnv* Env = FAndroidApplication::GetJavaEnv(true)) {
            jstring str_item_id = Env->NewStringUTF(item_id.c_str());
            jstring str_pass_through = Env->NewStringUTF(pass_through_param.c_str());
            FJavaWrapper::CallVoidMethod(Env, FJavaWrapper::GameActivityThis, SamsungIAP_startPayment, str_item_id, str_pass_through, success_dialog);
            Env->DeleteLocalRef(str_pass_through);
            Env->DeleteLocalRef(str_item_id);
            UE_LOG(LogAndroid, Warning, TEXT("I found the java method SamsungIAP_startPayment\n"));
        }
        else
        {
            UE_LOG(LogAndroid, Warning, TEXT("ERROR Could note get Java ENV\n"));
        }

    }

    void IAPJniWrapper::consumePurchasedItems(const std::string& itemIDs) {
        if (JNIEnv* Env = FAndroidApplication::GetJavaEnv(true)) {
            jstring str = Env->NewStringUTF(itemIDs.c_str());
            FJavaWrapper::CallVoidMethod(Env, FJavaWrapper::GameActivityThis, SamsungIAP_consumePurchasedItems, str);
            Env->DeleteLocalRef(str);
            UE_LOG(LogAndroid, Warning, TEXT("I found the java method SamsungIAP_consumePurchasedItems\n"));
        }
        else
        {
            UE_LOG(LogAndroid, Warning, TEXT("ERROR Could note get Java ENV\n"));
        }
    }

    /********************************************
     * Listener
     ********************************************/

    void IAPJniWrapper::setListener(IAPListener* listener) {
        _listener = listener;
    }

    void IAPJniWrapper::removeListener(){
        _listener = nullptr;
    }
    
    IAPListener* IAPJniWrapper::getListener() {
        return _listener;
    }

    /********************************************
     * Callbacks
     ********************************************/

    void IAPJniWrapper::onGetProducts(int result, const FString& msg, const string& data) {
        UE_LOG(SamsungIAP_Native, Log, TEXT("%s"), *FString(data.c_str()));
        if(_listener != nullptr) {
            auto json_data = Json::parse(data);
            vector<ProductVo> products;

            for (auto& d : json_data.array_items()) {
                ProductVo p(d);
                products.push_back(p);
            }

            _listener->onGetProducts(result, msg, products);
        }
    }

    void IAPJniWrapper::onGetOwnedProducts(int result, const FString& msg, const string& data) {
        if(_listener != nullptr) {
            auto json_data = Json::parse(data);
            vector<OwnedProductVo> products;

            for(auto& d : json_data.array_items()) {
                OwnedProductVo p(d);
                products.push_back(p);
            }

            _listener->onGetOwnedProducts(result, msg, products);
        }
    }

    void IAPJniWrapper::onPayment(int result, const FString& msg, const string& data) {
        if(_listener != nullptr) {
            auto json_data = Json::parse(data);
            PurchaseVo purchase(json_data);
            _listener->onPayment(result, msg, purchase);
        }
    }

    void IAPJniWrapper::onConsumePurchasedItems(int result, const FString& msg, const string& data) {
        if(_listener != nullptr) {

            auto json_data = Json::parse(data);
            vector<ConsumeVo> products;

            for(auto& d : json_data.array_items()) {
                ConsumeVo p(d);
                products.push_back(p);
            }

            _listener->onConsumePurchasedItems(result, msg, products);
        }
    }


}

#else

//Stub function for other platforms
namespace samsung {

    static IAPJniWrapper* s_instance = nullptr;

    IAPJniWrapper* IAPJniWrapper::getInstance() {
        if(!s_instance) {
            s_instance = new IAPJniWrapper();
        }

        return s_instance;
    }

    IAPJniWrapper::IAPJniWrapper() {
        _listener = nullptr;
    }

    IAPJniWrapper::~IAPJniWrapper() {
    }

    void IAPJniWrapper::setOperationMode(int mode) {
        UE_LOG(SamsungIAP_Native, Warning, TEXT("Samsung IAP is not supported on this platform\n"));
    }

    void IAPJniWrapper::getProductDetails(const std::string& products) {
        UE_LOG(SamsungIAP_Native, Warning, TEXT("Samsung IAP is not supported on this platform\n"));
    }

    void IAPJniWrapper::getOwnedList(const std::string& type) {
        UE_LOG(SamsungIAP_Native, Warning, TEXT("Samsung IAP is not supported on this platform\n"));
    }

    void IAPJniWrapper::startPayment(const std::string& item_id, const std::string& pass_through_param, bool success_dialog) {
        UE_LOG(SamsungIAP_Native, Warning, TEXT("Samsung IAP is not supported on this platform\n"));
    }

    void IAPJniWrapper::consumePurchasedItems(const std::string& itemIDs) {
        UE_LOG(SamsungIAP_Native, Warning, TEXT("Samsung IAP is not supported on this platform\n"));
    }

    /********************************************
     * Listener
     ********************************************/

    void IAPJniWrapper::setListener(IAPListener* listener) {
        _listener = listener;
    }

    void IAPJniWrapper::removeListener(){
        _listener = nullptr;
    }
    
    IAPListener* IAPJniWrapper::getListener() {
        return _listener;
    }

    /********************************************
     * Callbacks
     ********************************************/

    void IAPJniWrapper::onGetProducts(int result, const FString& msg, const string& data) {
        UE_LOG(SamsungIAP_Native, Warning, TEXT("Samsung IAP is not supported on this platform\n"));
    }

    void IAPJniWrapper::onGetOwnedProducts(int result, const FString& msg, const string& data) {
        UE_LOG(SamsungIAP_Native, Warning, TEXT("Samsung IAP is not supported on this platform\n"));
    }

    void IAPJniWrapper::onPayment(int result, const FString& msg, const string& data) {
        UE_LOG(SamsungIAP_Native, Warning, TEXT("Samsung IAP is not supported on this platform\n"));
    }

    void IAPJniWrapper::onConsumePurchasedItems(int result, const FString& msg, const string& data) {
        UE_LOG(SamsungIAP_Native, Warning, TEXT("Samsung IAP is not supported on this platform\n"));
    }


}

#endif
