#pragma once

#include "CoreMinimal.h"
#include "Engine.h"
#include <string>
#include "json98.h"

#define IAP_MODE_TEST			1
#define IAP_MODE_TEST_FAILURE	-1
#define IAP_MODE_PRODUCTION		0

#define PRODUCT_TYPE_ITEM			"item"
#define PRODUCT_TYPE_SUBSCRIPTION	"subscription"
#define PRODUCT_TYPE_ALL			"all"

#define REQUEST_SUCCESS		1
#define REQUEST_FAILED		-1

namespace samsung {

	struct BaseVo
	{
  		BaseVo(const json98::Json& json);

		FString mItemId;
  		FString mItemName;
  		double mItemPrice;
  		FString mItemPriceString;
  		FString mCurrencyUnit;
  		FString mCurrencyCode;
  		FString mItemDesc;
  		FString mType;
  		bool mIsConsumable;
	};

	struct ProductVo : public BaseVo
	{
		ProductVo(const json98::Json& json);

		FString mSubscriptionDurationUnit;
		FString mSubscriptionDurationMultiplier;

		FString mItemImageUrl;
		FString mItemDownloadUrl;
		FString mReserved1;
		FString mReserved2;
		FString mFreeTrialPeriod;
	};

	struct OwnedProductVo : public BaseVo
	{
		OwnedProductVo(const json98::Json& json);

	    FString mPaymentId;
	    FString mPurchaseId;
	    long mPurchaseDate;
	    FString mPassThroughParam;

	    // Expiration date for a item which is "subscription" type
	    // ========================================================================
	    long mSubscriptionEndDate;
	    // ========================================================================
	};

	struct PurchaseVo : public BaseVo
	{
		PurchaseVo(const json98::Json& json);

		FString mPaymentId;
    	FString mPurchaseId;
    	long mPurchaseDate;
    	FString mVerifyUrl;
    	FString mPassThroughParam;

    	FString mItemImageUrl;
    	FString mItemDownloadUrl;
    	FString mReserved1;
    	FString mReserved2;
	};

	struct ConsumeVo
	{
		ConsumeVo(const json98::Json& json);

		FString mPurchaseId;
		FString mStatusString;
		FString mStatusCode;
	};

	class IAPListener
	{
	public:
		virtual void onGetProducts(int result, const FString& msg, const std::vector<ProductVo>& data) = 0;
		virtual void onGetOwnedProducts(int result, const FString& msg, const std::vector<OwnedProductVo>& data) = 0;
		virtual void onPayment(int result, const FString& msg, const PurchaseVo& data) = 0;
		virtual void onConsumePurchasedItems(int result, const FString& msg, const std::vector<ConsumeVo>& data) = 0;
	};

	class IAP
	{
	public:
		static void setOperationMode(int mode);
		static void getProductDetails(const FString& products);
		static void getOwnedList(const FString& type);
		static void startPayment(const FString& item_id, const FString& pass_through_param, bool show_success_dialog);
        static void consumePurchasedItems(const FString& item_ids);

		static void setListener(IAPListener* listener);
		static void removeListener();
		static IAPListener* getListener();
	};
}